@@ -10,6 +10,15 @@ plot.hhsmmdata <- function (x, ...)
 	Ns = c(0,cumsum(N))
 	xx = as.matrix(x$x)
 	d = ncol(xx)
+	if(anyNA(xx) | any(is.nan(xx))){
+		allmiss = which(apply(xx,1,function(t) all(is.na(t)|is.nan(t))))
+		for(ii in allmiss){
+			if(ii>1 & ii<nrow(xx))	xx[ii,] = (xx[ii-1,]+xx[ii+1,])/2
+			if(ii ==1) xx[ii,] = (xx[ii+2,]+xx[ii+1,])/2
+			if(ii == nrow(xx)) xx[ii,] = (xx[ii-1,]+xx[ii-2,])/2
+		}
+		xx = complete(mice(xx,printFlag=FALSE))
+	}
 	for(j in 1:d){
 		for(i in 1:length(N)){
 			if(d * length(N) <= 9){
